import stat
import subprocess

from debputy import elf_util
from debputy.filesystem_scan import FSPath
from debputy.packages import BinaryPackage

SKIPPED_DEBUG_DIRS = [
    'lib',
    'lib64',
    'usr',
    'bin',
    'sbin',
    'opt',
    'dev',
    'emul',
    '.build-id',
]

SKIP_DIRS = {
    f'./usr/lib/debug/{subdir}'
    for subdir in ['lib', 'lib64', 'usr', 'bin', 'sbin', 'opt', 'dev', 'emul', '.build-id']
}


def find_shlibdeps(binary_package: BinaryPackage,
                   fs_root: FSPath,
                   substvars_file: str):
    elf_files_to_process = []

    for fs_path, children in fs_root.walk():
        if fs_path.path in SKIP_DIRS:
            children.clear()
            continue
        if not fs_path.is_file or not fs_path.has_fs_path:
            continue

        st_obj = fs_path.stat()

        if not stat.S_ISREG(st_obj.st_mode) or st_obj.st_size < elf_util.ELF_HEADER_SIZE32:
            continue
        if not elf_util.is_so_or_exec_elf_file(fs_path.fs_path, assert_linking_type=elf_util.ELF_LINKING_TYPE_DYNAMIC):
            continue
        elf_files_to_process.append(fs_path.fs_path)

    if not elf_files_to_process:
        return

    dpkg_cmd = [
        'dpkg-shlibdeps',
        f'-T{substvars_file}'
    ]
    if binary_package.is_udeb:
        dpkg_cmd.append('-tudeb')
    dpkg_cmd.extend(elf_files_to_process)
    print(f"Executing for {binary_package.name}: {dpkg_cmd}")
    subprocess.check_call(dpkg_cmd)


